@@ -69,6 +69,7 @@ uint32_t ClingoAssignment::size()            const { return solver_->numVars();
 uint32_t ClingoAssignment::unassigned()      const { return solver_->numFreeVars(); }
 bool     ClingoAssignment::hasConflict()     const { return solver_->hasConflict(); }
 uint32_t ClingoAssignment::level()           const { return solver_->decisionLevel(); }
+uint32_t ClingoAssignment::rootLevel()       const { return solver_->rootLevel(); }
 bool     ClingoAssignment::hasLit(Lit_t lit) const { return solver_->validVar(decodeVar(lit)); }
 bool     ClingoAssignment::isTotal()         const { return solver_->numFreeVars() == 0u; }
 /////////////////////////////////////////////////////////////////////////////////////////
@@ -143,6 +144,7 @@ void ClingoPropagator::Control::removeWatch(Lit_t lit) {
 /////////////////////////////////////////////////////////////////////////////////////////
 // ClingoPropagator
 /////////////////////////////////////////////////////////////////////////////////////////
+static const uint32 CHECK_BIT = 31;
 // flags for clauses from propagator
 static const uint32 ccFlags_s[2] = {
 	/* 0: learnt */ ClauseCreator::clause_not_sat | Clasp::ClauseCreator::clause_int_lbd,
@@ -177,21 +179,33 @@ bool ClingoPropagator::inTrail(Literal p) const {
 	return std::find(trail_.begin(), trail_.end(), encodeLit(p)) != trail_.end();
 }
 
-void ClingoPropagator::registerUndo(Solver& s) {
+void ClingoPropagator::registerUndo(Solver& s, uint32 data) {
 	uint32 dl = s.decisionLevel();
 	if (dl != level_) {
 		POTASSCO_REQUIRE(dl > level_, "Stack property violated");
-		POTASSCO_REQUIRE(front_ == INT32_MAX || (dl - level_) == 1, "Propagate must be called on each level");
 		// first time we see this level
 		s.addUndoWatch(level_ = dl, this);
-		undo_.push_back(static_cast<uint32>(trail_.size()));
+		undo_.push_back(data);
+	}
+	else if (!undo_.empty() && data < undo_.back()) {
+		POTASSCO_ASSERT(test_bit(undo_.back(), CHECK_BIT));
+		// first time a watched literal is processed on this level
+		undo_.back() = data;
 	}
 }
+
+void ClingoPropagator::registerUndoCheck(Solver& s) {
+	if (uint32 dl = s.decisionLevel()) {
+		registerUndo(s, set_bit(s.decision(dl).var(), CHECK_BIT));
+	}
+}
+
 Constraint::PropResult ClingoPropagator::propagate(Solver& s, Literal p, uint32&) {
-	registerUndo(s);
+	registerUndo(s, static_cast<uint32>(trail_.size()));
 	trail_.push_back(encodeLit(p));
 	return PropResult(true, true);
 }
+
 void ClingoPropagator::undoLevel(Solver& s) {
 	POTASSCO_REQUIRE(s.decisionLevel() == level_, "Invalid undo");
 	uint32 beg = undo_.back();
@@ -201,26 +215,41 @@ void ClingoPropagator::undoLevel(Solver& s) {
 		ScopedLock(call_->lock(), call_->propagator(), Inc(epoch_))->undo(Control(*this, s), change);
 		prop_ = beg;
 	}
-	trail_.resize(beg);
-	if (front_ != INT32_MAX) {
+
+	if (front_ != INT32_MAX)
 		front_ = -1;
-		--level_;
+
+	if (!test_bit(beg, CHECK_BIT))
+		trail_.resize(beg);
+
+	if (!undo_.empty()) {
+		uint32 prev = undo_.back();
+		if (test_bit(prev, CHECK_BIT)) {
+			prev = clear_bit(prev, CHECK_BIT);
+		}
+		else {
+			POTASSCO_ASSERT(prev < trail_.size());
+			prev = decodeLit(trail_[prev]).var();
+		}
+		level_ = s.level(prev);
 	}
 	else {
-		level_ = !trail_.empty() ? s.level(decodeLit(trail_.back()).var()) : 0;
+		level_ = 0;
 	}
 }
+
 bool ClingoPropagator::propagateFixpoint(Clasp::Solver& s, Clasp::PostPropagator*) {
 	POTASSCO_REQUIRE(prop_ <= trail_.size(), "Invalid propagate");
 	for (Control ctrl(*this, s, state_prop); prop_ != trail_.size() || front_ < (int32)s.numAssignedVars();) {
 		if (prop_ != trail_.size()) {
 			// create copy because trail might change during call to user propagation
 			temp_.assign(trail_.begin() + prop_, trail_.end());
+			POTASSCO_REQUIRE(s.level(decodeLit(temp_[0]).var()) == s.decisionLevel(), "Propagate must be called on each level");
 			prop_ = static_cast<uint32>(trail_.size());
 			ScopedLock(call_->lock(), call_->propagator(), Inc(epoch_))->propagate(ctrl, Potassco::toSpan(temp_));
 		}
 		else {
-			registerUndo(s);
+			registerUndoCheck(s);
 			front_ = (int32)s.numAssignedVars();
 			ScopedLock(call_->lock(), call_->propagator(), Inc(epoch_))->check(ctrl);
 		}
@@ -354,7 +383,7 @@ ClingoPropagatorInit::ClingoPropagatorInit(Potassco::AbstractPropagator& cb, Cli
 }
 ClingoPropagatorInit::~ClingoPropagatorInit()       { delete history_; }
 void ClingoPropagatorInit::prepare(SharedContext&)  {}
-bool ClingoPropagatorInit::addPost(Solver& s)       { return s.addPost(new ClingoPropagator(this)); }
+bool ClingoPropagatorInit::applyConfig(Solver& s)   { return s.addPost(new ClingoPropagator(this)); }
 void ClingoPropagatorInit::unfreeze(SharedContext&) {
 	if (history_) {
 		for (ChangeList::const_iterator it = changes_.begin(), end = changes_.end(); it != end; ++it) {
@@ -416,4 +445,50 @@ void ClingoPropagatorInit::enableHistory(bool b) {
 	else if (!history_) { history_ = new History(); }
 }
 
+/////////////////////////////////////////////////////////////////////////////////////////
+// ClingoHeuristic
+/////////////////////////////////////////////////////////////////////////////////////////
+ClingoHeuristic::ClingoHeuristic(Potassco::AbstractHeuristic& clingoHeuristic, DecisionHeuristic* claspHeuristic, ClingoPropagatorLock* lock)
+	: clingo_(&clingoHeuristic)
+	, clasp_(claspHeuristic)
+	, lock_(lock) {}
+
+Literal ClingoHeuristic::doSelect(Solver& s) {
+	typedef Scoped<Potassco::AbstractHeuristic, ClingoPropagatorLock, &ClingoPropagatorLock::lock, &ClingoPropagatorLock::unlock, Nop> ScopedLock;
+	Literal fallback = clasp_->doSelect(s);
+	if (s.hasConflict())
+		return fallback;
+
+	ClingoAssignment assignment(s);
+	Potassco::Lit_t lit = ScopedLock(lock_, clingo_)->decide(s.id(), assignment, encodeLit(fallback));
+	Literal decision = lit != 0 ? decodeLit(lit) : fallback;
+	return s.validVar(decision.var()) && !s.isFalse(decision) ? decision : fallback;
+}
+
+void ClingoHeuristic::startInit(const Solver& s)    { clasp_->startInit(s); }
+void ClingoHeuristic::endInit(Solver& s)            { clasp_->endInit(s); }
+void ClingoHeuristic::detach(Solver& s)             { if (clasp_.is_owner()) { clasp_->detach(s); } }
+void ClingoHeuristic::setConfig(const HeuParams& p) { clasp_->setConfig(p); }
+void ClingoHeuristic::newConstraint(const Solver& s, const Literal* p, LitVec::size_type sz, ConstraintType t) {
+	clasp_->newConstraint(s, p, sz, t);
+}
+
+void ClingoHeuristic::updateVar(const Solver& s, Var v, uint32 n)                   { clasp_->updateVar(s, v, n); }
+void ClingoHeuristic::simplify(const Solver& s, LitVec::size_type st)               { clasp_->simplify(s, st); }
+void ClingoHeuristic::undoUntil(const Solver& s, LitVec::size_type st)              { clasp_->undoUntil(s, st); }
+void ClingoHeuristic::updateReason(const Solver& s, const LitVec& x, Literal r)     { clasp_->updateReason(s, x, r); }
+bool ClingoHeuristic::bump(const Solver& s, const WeightLitVec& w, double d)        { return clasp_->bump(s, w, d); }
+Literal ClingoHeuristic::selectRange(Solver& s, const Literal* f, const Literal* l) { return clasp_->selectRange(s, f, l); }
+
+
+DecisionHeuristic* ClingoHeuristic::fallback() const { return clasp_.get();  }
+
+ClingoHeuristic::Factory::Factory(Potassco::AbstractHeuristic& clingoHeuristic, ClingoPropagatorLock* lock)
+	: clingo_(&clingoHeuristic)
+	, lock_(lock) {}
+
+DecisionHeuristic* ClingoHeuristic::Factory::create(Heuristic_t::Type t, const HeuParams& p) {
+	return new ClingoHeuristic(*clingo_, Heuristic_t::create(t, p), lock_);
+}
+
 }
