@@ -627,7 +627,7 @@ TEST_CASE("Facade", "[facade]") {
 	SECTION("testUserConfigurator") {
 		struct MyAddPost : public ClaspConfig::Configurator {
 			MyAddPost() : called(false) {}
-			virtual bool addPost(Solver&) { return called = true; }
+			virtual bool applyConfig(Solver&) { return called = true; }
 			bool called;
 		} myAddPost;
 		config.addConfigurator(&myAddPost);
@@ -636,10 +636,10 @@ TEST_CASE("Facade", "[facade]") {
 		REQUIRE(myAddPost.called);
 	}
 	SECTION("testUserHeuristic") {
-		struct MyHeu {
-			static DecisionHeuristic* creator(Heuristic_t::Type, const HeuParams&) { throw MyHeu(); }
+		struct MyHeu : BasicSatConfig::HeuristicCreator {
+			DecisionHeuristic* create(Heuristic_t::Type, const HeuParams&) { throw MyHeu(); }
 		};
-		config.setHeuristicCreator(&MyHeu::creator);
+		config.setHeuristicCreator(new MyHeu(), Ownership_t::Acquire);
 		lpAdd(libclasp.startAsp(config, true), "{x1}.");
 		REQUIRE_THROWS_AS(libclasp.prepare(), MyHeu);
 	}
@@ -708,6 +708,25 @@ TEST_CASE("Incremental solving", "[facade]") {
 #if CLASP_HAS_THREADS
 
 TEST_CASE("Facade mt", "[facade][mt]") {
+	struct EventVar {
+		EventVar() : fired(false) {}
+		void fire() {
+			{
+				Clasp::mt::unique_lock<Clasp::mt::mutex> lock(mutex);
+				fired = true;
+			}
+			cond.notify_all();
+		}
+		void wait() {
+			for (Clasp::mt::unique_lock<Clasp::mt::mutex> lock(mutex); !fired;) {
+				cond.wait(lock);
+			}
+		}
+		Clasp::mt::mutex mutex;
+		Clasp::mt::condition_variable cond;
+		bool fired;
+	};
+
 	Clasp::ClaspFacade libclasp;
 	Clasp::ClaspConfig config;
 	typedef ClaspFacade::SolveHandle AsyncResult;
@@ -767,6 +786,39 @@ TEST_CASE("Facade mt", "[facade][mt]") {
 		AsyncResult it = libclasp.solve(SolveMode_t::Async|SolveMode_t::Yield);
 		REQUIRE(it.get().unsat());
 	}
+	SECTION("testSolveWinnerMt") {
+		struct Blocker : public PostPropagator {
+			Blocker(EventVar& ev) : eventVar(&ev) {}
+			uint32 priority() const { return PostPropagator::priority_reserved_ufs; }
+			virtual bool propagateFixpoint(Solver&, Clasp::PostPropagator*) { return true; }
+			virtual bool isModel(Solver&) { eventVar->wait(); return true; }
+			EventVar* eventVar;
+		};
+		struct Unblocker : EventHandler {
+			Unblocker(EventVar& ev) : eventVar(&ev) {}
+			virtual bool onModel(const Solver&, const Model&) { eventVar->fire(); return false; }
+			EventVar* eventVar;
+		};
+		config.solve.numModels = 0;
+		config.solve.enumMode = EnumOptions::enum_record;
+		config.solve.algorithm.threads = 4;
+		lpAdd(libclasp.startAsp(config, true), "{x1;x2;x3;x4}.");
+		EventVar eventVar;
+		libclasp.prepare();
+		uint32 expectedWinner = 0;
+		SECTION("Solver 3") { expectedWinner = 3; }
+		SECTION("Solver 1") { expectedWinner = 1; }
+		SECTION("Solver 2") { expectedWinner = 2; }
+		SECTION("Solver 0") { expectedWinner = 0; }
+		for (uint32 i = 0; i != libclasp.ctx.concurrency(); ++i) {
+			if (i != expectedWinner)
+				libclasp.ctx.solver(i)->addPost(new Blocker(eventVar));
+		}
+		Unblocker unblocker(eventVar);
+		AsyncResult result = libclasp.solve(SolveMode_t::Async, LitVec(), &unblocker);
+		uint32 winner = result.waitFor(5.0) ? libclasp.ctx.winner() : (eventVar.fire(), result.wait(), 0xFEE1DEAD);
+		REQUIRE(winner == expectedWinner);
+	}
 	SECTION("testInterruptBeforeSolve") {
 		config.solve.numModels = 0;
 		lpAdd(libclasp.startAsp(config, true), "{x1}.");
@@ -880,24 +932,7 @@ TEST_CASE("Facade mt", "[facade][mt]") {
 		}
 	}
 	SECTION("test mt exception handling") {
-		struct EventVar {
-			EventVar() : fired(false) {}
-			void fire() {
-				{
-					Clasp::mt::unique_lock<Clasp::mt::mutex> lock(mutex);
-					fired = true;
-				}
-				cond.notify_one();
-			}
-			void wait() {
-				for (Clasp::mt::unique_lock<Clasp::mt::mutex> lock(mutex); !fired;) {
-					cond.wait(lock);
-				}
-			}
-			Clasp::mt::mutex mutex;
-			Clasp::mt::condition_variable cond;
-			bool fired;
-		} ev;
+		EventVar ev;
 		config.solve.numModels = 0;
 		config.solve.setSolvers(2);
 		lpAdd(libclasp.startAsp(config, true), "{x1;x2}.");
@@ -1284,6 +1319,20 @@ struct PropagatorTest {
 	SharedContext ctx;
 	VarVec v;
 };
+
+struct DebugLock : ClingoPropagatorLock {
+	DebugLock() : locked(false) {}
+	virtual void lock() {
+		REQUIRE(!locked);
+		locked = true;
+	}
+	virtual void unlock() {
+		REQUIRE(locked);
+		locked = false;
+	}
+	bool locked;
+};
+
 }
 TEST_CASE("Clingo propagator", "[facade][propagator]") {
 	typedef ClingoPropagatorInit MyInit;
@@ -1321,7 +1370,7 @@ TEST_CASE("Clingo propagator", "[facade][propagator]") {
 		prop.v1 = encodeLit(posLit(v[1]));
 		prop.v2 = encodeLit(posLit(v[2]));
 		MyInit pp(prop);
-		pp.addPost(*ctx.master());
+		pp.applyConfig(*ctx.master());
 		ctx.endInit();
 		ctx.master()->assume(posLit(v[1])) && ctx.master()->propagate();
 		ctx.master()->assume(negLit(v[2])) && ctx.master()->propagate();
@@ -1336,7 +1385,7 @@ TEST_CASE("Clingo propagator", "[facade][propagator]") {
 		tp.addWatch(posLit(v[3]));
 		tp.addWatch(negLit(v[3]));
 		tp.addWatch(negLit(v[4]));
-		tp.addPost(*ctx.master());
+		tp.applyConfig(*ctx.master());
 		ctx.endInit();
 		Solver& s = *ctx.master();
 		s.assume(posLit(v[1])) && s.propagate();
@@ -1358,7 +1407,7 @@ TEST_CASE("Clingo propagator", "[facade][propagator]") {
 		tp.addWatch(prop.fire = negLit(v[3]));
 		prop.addToClause(posLit(v[1]));
 		prop.addToClause(posLit(v[2]));
-		tp.addPost(*ctx.master());
+		tp.applyConfig(*ctx.master());
 		ctx.endInit();
 		Solver& s = *ctx.master();
 		s.assume(negLit(v[3])) && s.propagate();
@@ -1369,7 +1418,7 @@ TEST_CASE("Clingo propagator", "[facade][propagator]") {
 		tp.addWatch(prop.fire = negLit(v[3]));
 		prop.addToClause(posLit(v[1]));
 		prop.addToClause(posLit(v[2]));
-		tp.addPost(*ctx.master());
+		tp.applyConfig(*ctx.master());
 		ctx.endInit();
 		Solver& s = *ctx.master();
 		s.assume(negLit(v[2])) && s.propagate();
@@ -1386,7 +1435,7 @@ TEST_CASE("Clingo propagator", "[facade][propagator]") {
 		prop.addToClause(posLit(v[1]));
 		prop.addToClause(posLit(v[2]));
 		prop.addToClause(posLit(v[3]));
-		tp.addPost(*ctx.master());
+		tp.applyConfig(*ctx.master());
 		ctx.endInit();
 		Solver& s = *ctx.master();
 		s.assume(negLit(v[1])) && s.propagate();
@@ -1403,7 +1452,7 @@ TEST_CASE("Clingo propagator", "[facade][propagator]") {
 		tp.addWatch(prop.fire = negLit(v[3]));
 		prop.addToClause(posLit(v[1]));
 		prop.addToClause(posLit(v[2]));
-		tp.addPost(*ctx.master());
+		tp.applyConfig(*ctx.master());
 		ctx.endInit();
 		Solver& s = *ctx.master();
 		s.assume(negLit(v[2])) && s.propagate();
@@ -1427,7 +1476,7 @@ TEST_CASE("Clingo propagator", "[facade][propagator]") {
 		test.addVars(1);
 		tp.addWatch(prop.fire = negLit(v[1]));
 		prop.addToClause(negLit(0));
-		tp.addPost(*ctx.master());
+		tp.applyConfig(*ctx.master());
 		ctx.endInit();
 		Solver& s = *ctx.master();
 		s.assume(negLit(v[1]));
@@ -1438,7 +1487,7 @@ TEST_CASE("Clingo propagator", "[facade][propagator]") {
 		tp.addWatch(prop.fire = negLit(v[3]));
 		prop.addToClause(posLit(v[1]));
 		prop.addToClause(posLit(v[2]));
-		tp.addPost(*ctx.master());
+		tp.applyConfig(*ctx.master());
 		ctx.endInit();
 		Solver& s = *ctx.master();
 		s.assume(posLit(v[1])) && s.force(negLit(v[2]), posLit(v[1])) && s.propagate();
@@ -1451,7 +1500,7 @@ TEST_CASE("Clingo propagator", "[facade][propagator]") {
 		test.addVars(3);
 		prop.addToClause(posLit(v[1]));
 		prop.addToClause(posLit(v[3]));
-		tp.addPost(*ctx.master());
+		tp.applyConfig(*ctx.master());
 		ctx.endInit();
 		Solver& s = *ctx.master();
 		ValueRep v = s.search(-1, -1);
@@ -1462,7 +1511,7 @@ TEST_CASE("Clingo propagator", "[facade][propagator]") {
 		test.addVars(3);
 		prop.addToClause(negLit(v[1]));
 		prop.addToClause(negLit(v[2]));
-		tp.addPost(*ctx.master());
+		tp.applyConfig(*ctx.master());
 		ctx.endInit();
 		Solver& s = *ctx.master();
 		s.assume(posLit(v[1]));
@@ -1482,7 +1531,7 @@ TEST_CASE("Clingo propagator", "[facade][propagator]") {
 		prop.fire = lit_true();
 		prop.clProp = Potassco::Clause_t::Static;
 		tp.addWatch(negLit(v[1]));
-		tp.addPost(*ctx.master());
+		tp.applyConfig(*ctx.master());
 		ctx.endInit();
 
 		Solver& s = *ctx.master();
@@ -1811,21 +1860,11 @@ TEST_CASE("Clingo propagator", "[facade][propagator]") {
 
 TEST_CASE("Clingo propagator init", "[facade][propagator]") {
 	typedef ClingoPropagatorInit MyInit;
-	struct DebugLock : ClingoPropagatorLock {
-		DebugLock() : locked(false) {}
-		virtual void lock() {
-			REQUIRE(!locked);
-			locked = true;
-		}
-		virtual void unlock() {
-			REQUIRE(locked);
-			locked = false;
-		}
-		bool locked;
-	} debugLock;
+
 	PropagatorTest test;
 	SharedContext& ctx = test.ctx;
 	MyProp         prop;
+	DebugLock      debugLock;
 	MyInit         init(prop, &debugLock);
 
 	test.addVars(5);
@@ -1835,7 +1874,7 @@ TEST_CASE("Clingo propagator init", "[facade][propagator]") {
 		init.addWatch(posLit(1));
 		init.addWatch(posLit(2));
 		init.addWatch(posLit(4));
-		init.addPost(s0);
+		init.applyConfig(s0);
 		ctx.endInit();
 		PostPropagator* pp = s0.getPost(PostPropagator::priority_class_general);
 		REQUIRE(s0.hasWatch(posLit(1), pp));
@@ -1846,7 +1885,7 @@ TEST_CASE("Clingo propagator init", "[facade][propagator]") {
 	SECTION("ignore duplicate watches from init") {
 		init.addWatch(posLit(1));
 		init.addWatch(posLit(1));
-		init.addPost(s0);
+		init.applyConfig(s0);
 		ctx.endInit();
 		PostPropagator* pp = s0.getPost(PostPropagator::priority_class_general);
 		REQUIRE(s0.hasWatch(posLit(1), pp));
@@ -1856,7 +1895,7 @@ TEST_CASE("Clingo propagator init", "[facade][propagator]") {
 	SECTION("ignore duplicates on solver-specific init") {
 		init.addWatch(posLit(1));
 		init.addWatch(0, posLit(1));
-		init.addPost(s0);
+		init.applyConfig(s0);
 		ctx.endInit();
 		PostPropagator* pp = s0.getPost(PostPropagator::priority_class_general);
 		REQUIRE(s0.hasWatch(posLit(1), pp));
@@ -1869,8 +1908,8 @@ TEST_CASE("Clingo propagator init", "[facade][propagator]") {
 		init.addWatch(posLit(1));     // add to both
 		init.addWatch(0, posLit(2));
 		init.addWatch(1, posLit(3));
-		init.addPost(s0);
-		init.addPost(s1);
+		init.applyConfig(s0);
+		init.applyConfig(s1);
 		ctx.endInit(true);
 		PostPropagator* pp0 = s0.getPost(PostPropagator::priority_class_general);
 		PostPropagator* pp1 = s1.getPost(PostPropagator::priority_class_general);
@@ -1892,8 +1931,8 @@ TEST_CASE("Clingo propagator init", "[facade][propagator]") {
 		init.addWatch(posLit(2));
 		init.addWatch(posLit(3));
 		init.removeWatch(1, posLit(2));
-		init.addPost(s0);
-		init.addPost(s1);
+		init.applyConfig(s0);
+		init.applyConfig(s1);
 		ctx.endInit(true);
 		PostPropagator* pp0 = s0.getPost(PostPropagator::priority_class_general);
 		PostPropagator* pp1 = s1.getPost(PostPropagator::priority_class_general);
@@ -1910,7 +1949,7 @@ TEST_CASE("Clingo propagator init", "[facade][propagator]") {
 		init.addWatch(posLit(1));
 		init.removeWatch(0, posLit(1));
 		init.addWatch(posLit(1));
-		init.addPost(s0);
+		init.applyConfig(s0);
 		ctx.endInit();
 		PostPropagator* pp = s0.getPost(PostPropagator::priority_class_general);
 		REQUIRE(s0.hasWatch(posLit(1), pp));
@@ -1920,7 +1959,7 @@ TEST_CASE("Clingo propagator init", "[facade][propagator]") {
 		init.addWatch(posLit(1));
 		ctx.startAddConstraints();
 		ctx.addUnary(posLit(1));
-		init.addPost(s0);
+		init.applyConfig(s0);
 		ctx.endInit();
 		PostPropagator* pp = s0.getPost(PostPropagator::priority_class_general);
 		REQUIRE(prop.change.size() == 1);
@@ -1932,7 +1971,7 @@ TEST_CASE("Clingo propagator init", "[facade][propagator]") {
 		ctx.startAddConstraints();
 		ctx.addUnary(posLit(1));
 		s0.propagate();
-		init.addPost(s0);
+		init.applyConfig(s0);
 		ctx.endInit();
 		PostPropagator* pp = s0.getPost(PostPropagator::priority_class_general);
 		REQUIRE(prop.change.size() == 1);
@@ -1943,7 +1982,7 @@ TEST_CASE("Clingo propagator init", "[facade][propagator]") {
 		init.addWatch(posLit(1));
 		ctx.startAddConstraints();
 		ctx.addUnary(posLit(1));
-		init.addPost(s0);
+		init.applyConfig(s0);
 		ctx.endInit();
 		PostPropagator* pp = s0.getPost(PostPropagator::priority_class_general);
 		REQUIRE(prop.change.size() == 1);
@@ -1974,8 +2013,8 @@ TEST_CASE("Clingo propagator init", "[facade][propagator]") {
 		init.removeWatch(2, posLit(1));
 		init.removeWatch(2, posLit(3));
 		init.addWatch(2, posLit(4));
-		init.addPost(s0);
-		init.addPost(s1);
+		init.applyConfig(s0);
+		init.applyConfig(s1);
 		// don't add s2 yet
 		ctx.endInit(true);
 
@@ -1985,7 +2024,7 @@ TEST_CASE("Clingo propagator init", "[facade][propagator]") {
 		init.prepare(ctx);
 		ctx.startAddConstraints();
 		init.addWatch(posLit(5));
-		init.addPost(s2);
+		init.applyConfig(s2);
 		ctx.endInit(true);
 		PostPropagator* pp2 = s2.getPost(PostPropagator::priority_class_general);
 
@@ -2000,7 +2039,7 @@ TEST_CASE("Clingo propagator init", "[facade][propagator]") {
 		class Prop : public Potassco::AbstractPropagator {
 		public:
 			Prop() {}
-			virtual void propagate(Potassco::AbstractSolver& s, const ChangeList&) {}
+			virtual void propagate(Potassco::AbstractSolver&, const ChangeList&) {}
 			virtual void undo(const Potassco::AbstractSolver&, const ChangeList&)  {}
 			virtual void check(Potassco::AbstractSolver& s) {
 				while (!add.empty()) {
@@ -2023,7 +2062,7 @@ TEST_CASE("Clingo propagator init", "[facade][propagator]") {
 		SECTION("ignore watches already added in init") {
 			init.addWatch(posLit(1));
 			prop.addWatch(posLit(1));
-			init.addPost(s0);
+			init.applyConfig(s0);
 			ctx.endInit();
 			PostPropagator* pp = s0.getPost(PostPropagator::priority_class_general);
 			REQUIRE(s0.hasWatch(posLit(1), pp));
@@ -2034,7 +2073,7 @@ TEST_CASE("Clingo propagator init", "[facade][propagator]") {
 
 		SECTION("ignore watches in init already added during solving") {
 			prop.addWatch(posLit(1));
-			init.addPost(s0);
+			init.applyConfig(s0);
 			ctx.endInit();
 			PostPropagator* pp = s0.getPost(PostPropagator::priority_class_general);
 			REQUIRE(prop.add.empty());
@@ -2053,7 +2092,7 @@ TEST_CASE("Clingo propagator init", "[facade][propagator]") {
 		SECTION("remove watch during solving") {
 			init.addWatch(posLit(1));
 			prop.removeWatch(posLit(1));
-			init.addPost(s0);
+			init.applyConfig(s0);
 			ctx.endInit();
 			PostPropagator* pp = s0.getPost(PostPropagator::priority_class_general);
 			ctx.unfreeze();
@@ -2066,7 +2105,7 @@ TEST_CASE("Clingo propagator init", "[facade][propagator]") {
 		SECTION("remove watch during solving then add on init") {
 			init.addWatch(posLit(1));
 			prop.removeWatch(posLit(1));
-			init.addPost(s0);
+			init.applyConfig(s0);
 			ctx.endInit();
 			PostPropagator* pp = s0.getPost(PostPropagator::priority_class_general);
 			ctx.unfreeze();
@@ -2080,7 +2119,7 @@ TEST_CASE("Clingo propagator init", "[facade][propagator]") {
 
 		SECTION("add watch during solving then remove on init") {
 			prop.addWatch(posLit(1));
-			init.addPost(s0);
+			init.applyConfig(s0);
 			ctx.endInit();
 			PostPropagator* pp = s0.getPost(PostPropagator::priority_class_general);
 			ctx.unfreeze();
@@ -2094,4 +2133,115 @@ TEST_CASE("Clingo propagator init", "[facade][propagator]") {
 	}
 }
 
+TEST_CASE("Clingo heuristic", "[facade][heuristic]") {
+	class ClingoHeu : public Potassco::AbstractHeuristic {
+	public:
+		ClingoHeu() : lock(0) {}
+		virtual Lit decide(Potassco::Id_t, const Potassco::AbstractAssignment& assignment, Lit fallback) {
+			REQUIRE((!lock || lock->locked));
+			REQUIRE(!assignment.isTotal());
+			REQUIRE(assignment.value(fallback) == Potassco::Value_t::Free);
+			fallbacks.push_back(fallback);
+			for (Potassco::Atom_t i = Potassco::atom(fallback) + 1;; ++i) {
+				if (!assignment.hasLit(i))
+					i = 1;
+				if (assignment.value(i) == Potassco::Value_t::Free) {
+					selected.push_back(Potassco::neg(i));
+					return selected.back();
+				}
+			}
+		}
+		Potassco::LitVec selected;
+		Potassco::LitVec fallbacks;
+		DebugLock*       lock;
+	};
+	ClaspConfig config;
+	ClaspFacade libclasp;
+	ClingoHeu   heuristic;
+	SECTION("Factory") {
+		config.setHeuristicCreator(new ClingoHeuristic::Factory(heuristic));
+		DecisionHeuristic* heu = config.heuristic(0);
+		REQUIRE(dynamic_cast<ClingoHeuristic*>(heu) != 0);
+		REQUIRE(dynamic_cast<ClaspBerkmin*>(dynamic_cast<ClingoHeuristic*>(heu)->fallback()) != 0);
+		delete heu;
+	}
+
+	SECTION("Clingo heuristic is called with fallback") {
+		SolverParams& opts = config.addSolver(0);
+		opts.heuId    = Heuristic_t::None;
+		config.setHeuristicCreator(new ClingoHeuristic::Factory(heuristic));
+		Clasp::Asp::LogicProgram& asp = libclasp.startAsp(config);
+		lpAdd(asp, "{x1;x2;x3}.");
+		asp.endProgram();
+		libclasp.prepare();
+
+		SingleOwnerPtr<DecisionHeuristic> fallback(Heuristic_t::create(Heuristic_t::None, HeuParams()));
+		Solver& s = *libclasp.ctx.master();
+
+		while (s.numFreeVars() != 0) {
+			Literal fb = fallback->doSelect(s);
+			Literal lit = s.heuristic()->doSelect(s);
+			REQUIRE(lit == decodeLit(heuristic.selected.back()));
+			REQUIRE(fb  == decodeLit(heuristic.fallbacks.back()));
+			s.assume(lit) && s.propagate();
+		}
+
+		REQUIRE(heuristic.selected.size() == s.numVars());
+	}
+
+	SECTION("Restricted lookahead decorates clingo heuristic") {
+		SolverParams& opts = config.addSolver(0);
+		opts.lookOps  = 2;
+		opts.lookType = 1;
+		opts.heuId    = Heuristic_t::Vsids;
+		config.setHeuristicCreator(new ClingoHeuristic::Factory(heuristic));
+		Clasp::Asp::LogicProgram& asp = libclasp.startAsp(config);
+		lpAdd(asp, "{x1;x2;x3}.");
+		asp.endProgram();
+		libclasp.prepare();
+		DecisionHeuristic* heu = libclasp.ctx.master()->heuristic();
+		// heuristic is Restricted(Clingo(Vsids))
+		REQUIRE(dynamic_cast<ClingoHeuristic*>(heu) == 0);
+		REQUIRE(dynamic_cast<UnitHeuristic*>(heu) != 0);
+
+		// Restricted does not forward to its decorated heuristic
+		Literal lit = heu->doSelect(*libclasp.ctx.master());
+		REQUIRE(heuristic.fallbacks.empty());
+		REQUIRE(heuristic.selected.empty());
+		libclasp.ctx.master()->assume(lit);
+
+		// Last lookahead operation - disables restricted heuristic
+		libclasp.ctx.master()->propagate();
+
+		// Restricted is no longer enabled and should remove itself on this call
+		lit = heu->doSelect(*libclasp.ctx.master());
+		REQUIRE(heuristic.fallbacks.size() == 1);
+		REQUIRE(heuristic.selected.size() == 1);
+		REQUIRE(heuristic.selected.back() == encodeLit(lit));
+		REQUIRE(heuristic.fallbacks.back() != heuristic.selected.size());
+
+		// From now on, we only have Clingo(Vsids)
+		heu = libclasp.ctx.master()->heuristic();
+		REQUIRE(dynamic_cast<ClingoHeuristic*>(heu) != 0);
+		REQUIRE(dynamic_cast<ClaspVsids*>(dynamic_cast<ClingoHeuristic*>(heu)->fallback()) != 0);
+	}
+
+	SECTION("Heuristic is called under lock") {
+		DebugLock lock;
+		heuristic.lock = &lock;
+
+		config.setHeuristicCreator(new ClingoHeuristic::Factory(heuristic, &lock));
+		Clasp::Asp::LogicProgram& asp = libclasp.startAsp(config);
+		lpAdd(asp, "{x1;x2;x3}.");
+		asp.endProgram();
+		libclasp.prepare();
+
+		Solver& s = *libclasp.ctx.master();
+		s.decideNextBranch();
+		REQUIRE(!heuristic.selected.empty());
+		REQUIRE(!lock.locked);
+	}
+}
+
+
 } }
